@@ -5,54 +5,69 @@ Wrappers for lucene Fields and Documents.
 from future_builtins import map
 import datetime, calendar
 import operator
+import warnings
 import lucene
-try:
-    from java.lang import Double, Float, Long, Number, Object
-    from java.util import Arrays, HashSet
-    from org.apache.lucene import document, search, util
-    from org.apache.lucene.search import grouping
-except ImportError:
-    from lucene import Double, Float, Long, Number, Object, Arrays, HashSet
-    document = search = util = grouping = lucene
+from java.lang import Double, Float, Long, Number, Object
+from java.util import Arrays, HashSet
+from org.apache.lucene import document, index, search, util
+from org.apache.lucene.search import grouping
 from .queries import Query
 
-class Field(object):
+class Field(document.FieldType):
     """Saved parameters which can generate lucene Fields given values.
     
     :param name: name of field
-    :param store,index,termvector: field parameters, expressed as bools or strs, with lucene defaults
-    :param analyzed,omitNorms: additional index boolean settings
-    :param withPositions,withOffsets: additional termvector boolean settings
     :param boost: boost factor
-    :param attrs: additional attributes to set on the field
+    :param stored, indexed, settings: lucene FieldType attributes
     """
-    def __init__(self, name, store=False, index='analyzed', termvector=False, analyzed=False, omitNorms=False, \
-            withPositions=False, withOffsets=False, boost=1.0, **attrs):
-        self.name, self.boost, self.attrs = name, boost, attrs
-        if isinstance(store, bool):
-            store = 'yes' if store else 'no'
-        self.store = document.Field.Store.valueOf(store.upper())
-        if isinstance(index, bool):
-            self.index = document.Field.Index.toIndex(index, analyzed, omitNorms)
+    attrs = set(name[3].lower() + name[4:] for name in dir(document.FieldType) if name.startswith('set'))
+    def __init__(self, name, stored=False, indexed=True, boost=1.0, **settings):
+        document.FieldType.__init__(self)
+        self.name, self.boost = name, boost
+        if set(['store', 'index', 'termvector']).intersection(settings):
+            warnings.warn('lucene Field.{Store,Index,TermVector} deprecated, use FieldType attibutes', DeprecationWarning)
+            store = settings.get('store', False)
+            if isinstance(store, bool):
+                store = 'yes' if store else 'no'
+            store = document.Field.Store.valueOf(store.upper())
+            index = settings.get('index', 'analyzed')
+            if isinstance(index, bool):
+                index = document.Field.Index.toIndex(index, settings.get('analyzed', False), settings.get('omitNorms', False))
+            else:
+                index = document.Field.Index.valueOf(index.upper())
+            termvector = settings.get('termvector', False)
+            if isinstance(termvector, bool):
+                termvector = document.Field.TermVector.toTermVector(termvector, settings.get('withOffsets', False), settings.get('withPositions', False))
+            else:
+                termvector = document.Field.TermVector.valueOf(termvector.upper())
+            ft = document.Field.translateFieldType(store, index, termvector)
+            self.update(**Field.settings.fget(ft))
         else:
-            self.index = document.Field.Index.valueOf(index.upper())
-        if isinstance(termvector, bool):
-            self.termvector = document.Field.TermVector.toTermVector(termvector, withOffsets, withPositions)
-        else:
-            self.termvector = document.Field.TermVector.valueOf(termvector.upper())
-        next(Field.items(self, ' ')) # validate settings
+            self.update(stored=stored, indexed=indexed, **settings)
+    def update(self, docValueType='', indexOptions='', numericType='', **settings):
+        if docValueType:
+            self.setDocValueType(getattr(index.FieldInfo.DocValuesType, docValueType.upper()))
+        if indexOptions:
+            self.setIndexOptions(getattr(index.FieldInfo.IndexOptions, indexOptions.upper()))
+        if numericType:
+            self.setNumericType(getattr(document.FieldType.NumericType, numericType.upper()))
+        for name in settings:
+            getattr(self, 'set' + name[:1].upper() + name[1:])(settings[name])
+    @property
+    def settings(self):
+        "dict representation of settings"
+        defaults = document.FieldType()
+        result = {'indexed': self.indexed()}
+        for name in Field.attrs:
+            value = getattr(self, name)()
+            if value != getattr(defaults, name)():
+                result[name] = value if isinstance(value, int) else str(value)
+        return result
     def items(self, *values):
         "Generate lucene Fields suitable for adding to a document."
         for value in values:
-            if isinstance(value, basestring):
-                field = document.Field(self.name, value, self.store, self.index, self.termvector)
-            elif isinstance(value, lucene.JArray_byte):
-                field = document.Field(self.name, value)
-            else:
-                field = document.Field(self.name, value, self.termvector)
+            field = document.Field(self.name, value, self)
             field.setBoost(self.boost)
-            for name, value in self.attrs.items():
-                setattr(field, name, value)
             yield field
 
 class MapField(Field):
@@ -73,8 +88,10 @@ class NestedField(Field):
     
     :param sep: field separator used on name and values
     """
-    def __init__(self, name, sep='.', index=True, **kwargs):
-        Field.__init__(self, name, index=index, **kwargs)
+    def __init__(self, name, sep='.', tokenized=False, **kwargs):
+        if set(['store', 'index', 'termvector']).intersection(kwargs):
+            kwargs.setdefault('index', True)
+        Field.__init__(self, name, tokenized=tokenized, **kwargs)
         self.sep = sep
         self.names = tuple(self.values(name))
     def values(self, value):
@@ -84,12 +101,9 @@ class NestedField(Field):
             yield self.sep.join(value[:index])
     def items(self, *values):
         "Generate indexed component fields."
-        if self.store == document.Field.Store.YES:
-            for value in values:
-                yield document.Field(self.name, value, self.store, document.Field.Index.NO)
         for value in values:
             for index, text in enumerate(self.values(value)):
-                yield document.Field(self.names[index], text, document.Field.Store.NO, self.index, self.termvector)
+                yield document.Field(self.names[index], text, self)
     def prefix(self, value):
         "Return prefix query of the closest possible prefixed field."
         index = value.count(self.sep)
@@ -103,43 +117,34 @@ class NumericField(Field):
     """Field which indexes numbers in a prefix tree.
     
     :param name: name of field
-    :param step: precision step
+    :param type: optional int, float, or lucene NumericType string
     """
-    def __init__(self, name, step=None, store=False, index=True):
-        Field.__init__(self, name, store)
-        self.step = step or util.NumericUtils.PRECISION_STEP_DEFAULT
-        self.index = index
-        if hasattr(document, 'FieldType'):
-            self.type = document.FieldType()
-            self.type.setNumericPrecisionStep(self.step)
-            self.type.setStored(self.store == document.Field.Store.YES)
-            self.type.setIndexed(index)
+    def __init__(self, name, type=None, step=0, stored=False, tokenized=False, **kwargs):
+        if type:
+            kwargs['numericType'] = {int: 'long', float: 'double'}.get(type, str(type))
+        Field.__init__(self, name, stored=stored, tokenized=tokenized, **kwargs)
+        if step:
+            warnings.warn('step deprecated, use numericPrecisionStep', DeprecationWarning)
+            self.setNumericPrecisionStep(step)
     def items(self, *values):
         "Generate lucene NumericFields suitable for adding to a document."
+        if not self.numericType():
+            cls, = set(map(type, values))
+            self.update(numericType='double' if issubclass(cls, float) else 'long')
         for value in values:
-            if hasattr(self, 'type'):
-                if isinstance(value, float):
-                    self.type.setNumericType(document.FieldType.NumericType.DOUBLE)
-                    field = document.DoubleField(self.name, value, self.type)
-                else:
-                    self.type.setNumericType(document.FieldType.NumericType.LONG)
-                    field = document.LongField(self.name, long(value), self.type)
+            if isinstance(value, float):
+                yield document.DoubleField(self.name, value, self)
             else:
-                field = document.NumericField(self.name, self.step, self.store, self.index)
-                if isinstance(value, float):
-                    field.doubleValue = value
-                else:
-                    field.longValue = long(value)
-            yield field
+                yield document.LongField(self.name, long(value), self)
     def numeric(self, cls, start, stop, lower, upper):
         if isinstance(start, float) or isinstance(stop, float):
             start, stop = (value if value is None else Double(value) for value in (start, stop))
-            return cls.newDoubleRange(self.name, self.step, start, stop, lower, upper)
+            return cls.newDoubleRange(self.name, self.numericPrecisionStep(), start, stop, lower, upper)
         if start is not None:
             start = None if start < Long.MIN_VALUE else Long(long(start))
         if stop is not None:
             stop = None if stop > Long.MAX_VALUE else Long(long(stop))
-        return cls.newLongRange(self.name, self.step, start, stop, lower, upper)
+        return cls.newLongRange(self.name, self.numericPrecisionStep(), start, stop, lower, upper)
     def range(self, start, stop, lower=True, upper=False):
         "Return lucene NumericRangeQuery."
         return self.numeric(search.NumericRangeQuery, start, stop, lower, upper)
@@ -154,6 +159,8 @@ class DateTimeField(NumericField):
     """Field which indexes datetimes as a NumericField of timestamps.
     Supports datetimes, dates, and any prefix of time tuples.
     """
+    def __init__(self, name, **kwargs):
+        NumericField.__init__(self, name, type=float, **kwargs)
     def timestamp(self, date):
         "Return utc timestamp from date or time tuple."
         if isinstance(date, datetime.date):
@@ -200,10 +207,7 @@ class Document(dict):
     "Multimapping of field names to values, but default getters return the first value."
     def __init__(self, doc):
         for field in doc.getFields():
-            try:
-                value = field.binaryValue.string_ if field.binary else field.stringValue()
-            except AttributeError:
-                value = field.stringValue() or field.binaryValue() or field.numericValue().toString()
+            value = convert(field.numericValue() or field.stringValue() or field.binaryValue())
             self.setdefault(field.name(), []).append(value)
     def __getitem__(self, name):
         return dict.__getitem__(self, name)[0]
@@ -224,7 +228,7 @@ class Document(dict):
 
 def convert(value):
     "Return python object from java Object."
-    if hasattr(util, 'BytesRef') and util.BytesRef.instance_(value):
+    if util.BytesRef.instance_(value):
         return util.BytesRef.cast_(value).utf8ToString()
     if not Number.instance_(value):
         return value.toString() if Object.instance_(value) else value
@@ -261,7 +265,7 @@ class Hits(object):
         self.fields = fields
     def select(self, *fields):
         "Only load selected fields."
-        self.fields = getattr(document, 'MapFieldSelector', HashSet)(Arrays.asList(fields))
+        self.fields = HashSet(Arrays.asList(fields))
     def __len__(self):
         return len(self.scoredocs)
     def __getitem__(self, index):
@@ -272,12 +276,7 @@ class Hits(object):
             return type(self)(self.searcher, scoredocs, self.count, self.maxscore, self.fields)
         scoredoc = self.scoredocs[index]
         keys = search.FieldDoc.cast_(scoredoc).fields if search.FieldDoc.instance_(scoredoc) else ()
-        if self.fields is None:
-            doc = self.searcher.doc(scoredoc.doc)
-        elif hasattr(search.IndexSearcher, 'document'):
-            doc = self.searcher.document(scoredoc.doc, self.fields)
-        else:
-            doc = self.searcher.doc(scoredoc.doc, self.fields)
+        doc = self.searcher.doc(scoredoc.doc, self.fields)
         return Hit(doc, scoredoc.doc, scoredoc.score, keys)
     @property
     def ids(self):
@@ -318,16 +317,16 @@ class Grouping(object):
     :param count: maximum number of groups
     :param sort: lucene Sort to order groups
     """
-    collectors = getattr(grouping, 'term', grouping)
     def __init__(self, searcher, field, query=None, count=None, sort=None):
+        warnings.warn('Grouping deprecated, use GroupingSearch instead', DeprecationWarning)
         self.searcher, self.field = searcher, field
         self.query = query or search.MatchAllDocsQuery()
         self.sort = sort or search.Sort.RELEVANCE
         if count is None:
-            collector = self.collectors.TermAllGroupsCollector(field)
+            collector = grouping.term.TermAllGroupsCollector(field)
             search.IndexSearcher.search(self.searcher, self.query, collector)
             count = collector.groupCount
-        collector = self.collectors.TermFirstPassGroupingCollector(field, self.sort, count)
+        collector = grouping.term.TermFirstPassGroupingCollector(field, self.sort, count)
         search.IndexSearcher.search(self.searcher, self.query, collector)
         self.searchgroups = collector.getTopGroups(0, False).of_(grouping.SearchGroup)
     def __len__(self):
@@ -337,7 +336,7 @@ class Grouping(object):
             yield convert(searchgroup.groupValue)
     def facets(self, filter):
         "Generate field values and counts which match given filter."
-        collector = self.collectors.TermSecondPassGroupingCollector(self.field, self.searchgroups, self.sort, self.sort, 1, False, False, False)
+        collector = grouping.term.TermSecondPassGroupingCollector(self.field, self.searchgroups, self.sort, self.sort, 1, False, False, False)
         search.IndexSearcher.search(self.searcher, self.query, filter, collector)
         for groupdocs in collector.getTopGroups(0).groups:
             yield convert(groupdocs.groupValue), groupdocs.totalHits
@@ -352,9 +351,47 @@ class Grouping(object):
         sort = sort or self.sort
         if sort == search.Sort.RELEVANCE:
             scores = maxscore = True
-        collector = self.collectors.TermSecondPassGroupingCollector(self.field, self.searchgroups, self.sort, sort, count, scores, maxscore, True)
+        collector = grouping.term.TermSecondPassGroupingCollector(self.field, self.searchgroups, self.sort, sort, count, scores, maxscore, True)
         search.IndexSearcher.search(self.searcher, self.query, collector)
         for groupdocs in collector.getTopGroups(0).groups:
             hits = Hits(self.searcher, groupdocs.scoreDocs, groupdocs.totalHits, groupdocs.maxScore, getattr(self, 'fields', None))
             hits.value = convert(groupdocs.groupValue)
             yield hits
+
+class GroupingSearch(grouping.GroupingSearch):
+    """Inherited lucene GroupingSearch with optimized faceting.
+    
+    :param field: unique field name to group by
+    :param sort: lucene Sort to order groups and docs
+    :param cache: use unlimited caching
+    :param attrs: additional attributes to set
+    """
+    select = Hits.__dict__['select']
+    def __init__(self, field, sort=None, cache=True, **attrs):
+        grouping.GroupingSearch.__init__(self, field)
+        self.field = field
+        if sort:
+            self.groupSort = self.sortWithinGroup = sort
+            self.fillSortFields = True
+        if cache:
+            self.setCachingInMB(float('inf'), True)
+        for name in attrs:
+            getattr(type(self), name).__set__(self, attrs[name])
+    def __len__(self):
+        return self.allMatchingGroups.size()
+    def __iter__(self):
+        return map(convert, self.allMatchingGroups)
+    def search(self, searcher, query, filter, count):
+        if count is None:
+            count = sum(search.FieldCache.DEFAULT.getTermsIndex(reader, self.field).valueCount for reader in searcher.readers)
+        return grouping.GroupingSearch.search(self, searcher, filter, query, 0, count).groups
+    def facets(self, searcher, filter=None):
+        "Generate field values and counts which match given filter."
+        for groupdocs in self.search(searcher, search.MatchAllDocsQuery(), filter, None):
+            yield convert(groupdocs.groupValue), groupdocs.totalHits
+    def groups(self, searcher, query, filter=None, count=None):
+        "Generate grouped `Hits`_ from search parameters."
+        for groupdocs in self.search(searcher, query, filter, count):
+            hits = Hits(searcher, groupdocs.scoreDocs, groupdocs.totalHits, groupdocs.maxScore, getattr(self, 'fields', None))
+            hits.value = convert(groupdocs.groupValue)
+            yield hits
