@@ -50,15 +50,13 @@ import collections
 import itertools
 import os, optparse
 import contextlib
+import warnings
 try:
     import simplejson as json
 except ImportError:
     import json
 import lucene
-try:
-    from org.apache.lucene import search
-except ImportError:
-    search = lucene
+from org.apache.lucene import search
 import cherrypy
 try:
     from . import engine, client
@@ -175,10 +173,10 @@ class params:
     def fields(searcher, fields=None, **options):
         if fields is not None:
             fields = dict.fromkeys(fields)
-        multi = options.get('fields.multi', ())
+        multi = set(options.get('fields.multi', ()))
         indexed = (field.split(':') for field in options.get('fields.indexed', ()))
-        indexed = dict((item[0], searcher.comparator(*item)) for item in indexed)
-        return fields, multi, indexed
+        indexed = dict((item[0], searcher.comparator(*item, multi=item[0] in multi)) for item in indexed)
+        return fields, multi.difference(indexed), indexed
 
 def json_error(version, **body):
     "Transform errors into json format."
@@ -272,8 +270,7 @@ class WebSearcher(object):
             self.sync(host, path)
             cherrypy.response.status = httplib.ACCEPTED
         if isinstance(self.searcher, engine.MultiSearcher):
-            readers = map(engine.indexers.IndexReader, self.searcher.sequentialSubReaders)
-            return dict((reader.directory.toString(), reader.numDocs()) for reader in readers)
+            return dict((reader.directory().toString(), reader.numDocs()) for reader in self.searcher.indexReaders)
         return {self.searcher.directory.toString(): len(self.searcher)}
     @cherrypy.expose
     @cherrypy.tools.json_in(process_body=dict)
@@ -476,14 +473,14 @@ class WebSearcher(object):
                 terms[name][value] = list(itertools.islice(searcher.correct(name, value), spellcheck))
         return result
     @cherrypy.expose
-    @cherrypy.tools.params(count=int, step=int)
+    @cherrypy.tools.params(count=int, step=int, indexed=json.loads)
     def terms(self, name='', value=':', *path, **options):
         """Return data about indexed terms.
         
         **GET** /terms?
             Return field names, with optional selection.
             
-            &option=\ *chars*
+            &indexed=true|false
             
             :return: [*string*,... ]
         
@@ -722,7 +719,7 @@ class WebIndexer(WebSearcher):
         elif request.method == 'PUT':
             doc = getattr(request, 'json', {})
             with HTTPError(httplib.CONFLICT, KeyError, AssertionError):
-                assert self.indexer.fields[name].index.indexed, 'unique field must be indexed'
+                assert self.indexer.fields[name].indexed(), 'unique field must be indexed'
             with HTTPError(httplib.BAD_REQUEST, AssertionError):
                 assert doc.setdefault(name, value) == value, 'multiple values for unique field'
             self.indexer.update(name, value, doc)
@@ -752,7 +749,7 @@ class WebIndexer(WebSearcher):
     @cherrypy.tools.allow(paths=[('GET',), ('GET', 'PUT')])
     @cherrypy.tools.validate(on=False)
     def fields(self, name='', **settings):
-        """Return or store a field's parameters.
+        """Return or store a field's settings.
         
         **GET** /fields
             Return known field names.
@@ -760,11 +757,13 @@ class WebIndexer(WebSearcher):
             :return: [*string*,... ]
         
         **GET, PUT** /fields/*chars*
-            Set and return parameters for given field name.
+            Set and return settings for given field name.
             
-            {"store"|"index"|"termvector": *string*\|true|false,... }
+            {"stored"|"indexed"\|...: *string*\|true|false,... }
             
-            :return: {"store": *string*, "index": *string*, "termvector": *string*}
+            .. deprecated:: 1.5 lucene FieldType attributes used as settings
+            
+            :return: {"stored"|"indexed"\|...: *string*\|true|false,... }
         """
         if not name:
             return sorted(self.indexer.fields)
@@ -772,14 +771,18 @@ class WebIndexer(WebSearcher):
             if name not in self.indexer.fields:
                 cherrypy.response.status = httplib.CREATED
             with HTTPError(httplib.BAD_REQUEST, AttributeError):
-                self.indexer.set(name, **settings)
+                with warnings.catch_warnings(record=True) as messages:
+                    self.indexer.set(name, **settings)
+            for message in messages:
+                cherrypy.response.headers['warning'] = '199 lupyne "{0}"'.format(message.message)
         with HTTPError(httplib.NOT_FOUND, KeyError):
-            field = self.indexer.fields[name]
-        return dict((name, str(getattr(field, name))) for name in ['store', 'index', 'termvector'])
+            return self.indexer.fields[name].settings
 
 def init(vmargs='-Xrs,-Djava.awt.headless=true', **kwargs):
     "Callback to initialize VM and app roots after daemonizing."
-    lucene.initVM(vmargs=vmargs, **kwargs)
+    if vmargs:
+        kwargs['vmargs'] = vmargs
+    lucene.initVM(**kwargs)
     for app in cherrypy.tree.apps.values():
         if isinstance(app.root, WebSearcher):
             app.root.__init__(*app.root.__dict__.pop('args'), **app.root.__dict__.pop('kwargs'))
