@@ -9,21 +9,12 @@ import math
 import bisect
 import contextlib
 import lucene
-try:
-    from java.io import StringReader
-    from org.apache.lucene import analysis, document, search, store, util
-    from org.apache.lucene.analysis import miscellaneous, standard
-    from org.apache.lucene.search import grouping, highlight, vectorhighlight
-    from org.apache.pylucene.search import PythonFilter
-except ImportError:
-    from lucene import StringReader, PythonFilter
-    analysis = document = search = store = util = miscellaneous = standard = grouping = highlight = vectorhighlight = lucene
+from java.io import StringReader
+from org.apache.lucene import analysis, document, search, store, util
+from org.apache.lucene.search import grouping, highlight, vectorhighlight
+from org.apache.pylucene.search import PythonFilter
 from lupyne import engine
 from . import fixture
-if not hasattr(analysis, 'PorterStemFilter'):
-    analysis.PorterStemFilter = analysis.en.PorterStemFilter
-if hasattr(analysis, 'core'):
-    analysis.WhitespaceAnalyzer, analysis.WhitespaceTokenizer = analysis.core.WhitespaceAnalyzer, analysis.core.WhitespaceTokenizer
 
 class typeAsPayload(engine.TokenFilter):
     "Custom implementation of lucene TypeAsPayloadTokenFilter."
@@ -34,8 +25,8 @@ class typeAsPayload(engine.TokenFilter):
 def assertWarns(*categories):
     with warnings.catch_warnings(record=True) as messages:
         yield
-    for message, category in itertools.izip_longest(messages, categories):
-         assert issubclass(message.category, category), message
+    types = [message.category for message in messages]
+    assert types == list(categories), types
 
 class Filter(PythonFilter):
     "Broken filter to test errors are raised."
@@ -53,11 +44,11 @@ class TestCase(BaseTest):
     def testInterface(self):
         "Indexer and document interfaces."
         self.assertRaises(TypeError, engine.IndexSearcher)
-        analyzer = lambda reader: standard.StandardTokenizer(util.Version.values()[-1], reader)
-        stemmer = engine.Analyzer(analyzer, analysis.PorterStemFilter, typeAsPayload)
+        analyzer = lambda reader: analysis.standard.StandardTokenizer(util.Version.values()[-1], reader)
+        stemmer = engine.Analyzer(analyzer, analysis.en.PorterStemFilter, typeAsPayload)
         for token in stemmer.tokens('hello'):
             assert token.positionIncrement == 1
-            assert engine.TokenFilter(miscellaneous.EmptyTokenStream()).payload is None
+            assert engine.TokenFilter(analysis.miscellaneous.EmptyTokenStream()).payload is None
             assert token.term == 'hello'
             assert token.type == token.payload == '<ALPHANUM>'
             assert token.offset == (0, 5)
@@ -65,14 +56,14 @@ class TestCase(BaseTest):
             token.offset, token.positionIncrement = (0, 0), 0
         assert str(stemmer.parse('hellos', field=['body', 'title'])) == 'body:hello title:hello'
         assert str(stemmer.parse('hellos', field={'body': 1.0, 'title': 2.0})) == 'body:hello title:hello^2.0'
-        indexer = engine.Indexer(analyzer=stemmer, version=util.Version.LUCENE_30, writeLockTimeout=100L)
+        indexer = engine.Indexer(analyzer=stemmer, version=util.Version.LUCENE_43, writeLockTimeout=100L)
         assert indexer.config.writeLockTimeout == 100
         self.assertRaises(lucene.JavaError, engine.Indexer, indexer.directory)
         indexer.set('text')
-        indexer.set('name', store=True, index=False)
-        indexer.set('tag', store=True, index=True, boost=2.0)
+        indexer.set('name', stored=True, indexed=False)
+        indexer.set('tag', stored=True, tokenized=False, boost=2.0)
         for field in indexer.fields['tag'].items('sample'):
-            assert isinstance(field, document.Field) and getattr(field, 'getBoost', field.boost)() == 2.0
+            assert isinstance(field, document.Field) and field.boost() == 2.0
         searcher = indexer.indexSearcher
         indexer.commit()
         assert searcher is indexer.indexSearcher
@@ -95,16 +86,8 @@ class TestCase(BaseTest):
         assert not list(indexer.termvector(0, 'tag'))
         assert indexer.count('text', 'hello') == indexer.count('text:hello') == 1
         assert sorted(indexer.names()) == ['name', 'tag', 'text']
-        try:
-            names = indexer.names(indexed=True)
-        except AttributeError:
-            names = indexer.names('indexed', isIndexed=True)
-        assert sorted(names)[-2:] == ['tag', 'text']
-        try:
-            names = indexer.names(indexed=False)
-        except AttributeError:
-            names = indexer.names('unindexed', isIndexed=False)
-        assert 'name' in names
+        assert sorted(indexer.names(indexed=True)) == ['tag', 'text']
+        assert indexer.names(indexed=False) == ['name']
         assert list(indexer.terms('text')) == ['hello', 'world']
         assert list(indexer.terms('text', 'h', 'v')) == ['hello']
         assert dict(indexer.terms('text', 'w', counts=True)) == {'world': 1}
@@ -134,19 +117,14 @@ class TestCase(BaseTest):
         assert str(search.MatchAllDocsQuery() | query) == '*:* text:*'
         assert str(search.MatchAllDocsQuery() - query) == '*:* -text:*'
         query = +query
-        if hasattr(search.FuzzyQuery, 'defaultMaxEdits'):
-            query &= engine.Query.fuzzy('text', 'hello')
-            query |= engine.Query.fuzzy('text', 'hello', 1)
-            assert str(query) == '+text:* +text:hello~2 text:hello~1'
-        else:
-            query &= engine.Query.fuzzy('text', 'hello')
-            query |= engine.Query.fuzzy('text', 'hello', 0.1)
-            assert str(query) == '+text:* +text:hello~0.5 text:hello~0.1'
+        query &= engine.Query.fuzzy('text', 'hello')
+        query |= engine.Query.fuzzy('text', 'hello', 1)
+        assert str(query) == '+text:* +text:hello~2 text:hello~1'
         query = engine.Query.span('text', 'world')
         assert str(query.mask('name')) == 'mask(text:world) as name'
         assert str(query.payload()) == 'spanPayCheck(text:world, payloadRef: )'
-        assert isinstance(query.filter(cache=False), getattr(search, 'SpanQueryFilter', search.QueryWrapperFilter))
-        assert isinstance(query.filter(), getattr(search, 'CachingSpanFilter', search.CachingWrapperFilter))
+        assert isinstance(query.filter(cache=False), search.QueryWrapperFilter)
+        assert isinstance(query.filter(), search.CachingWrapperFilter)
         query = engine.Query.disjunct(0.1, query, name='sample')
         assert str(query) == '(text:world | name:sample)~0.1'
         query = engine.Query.near('text', 'hello', ('tag', 'python'), slop=-1, inOrder=False)
@@ -182,13 +160,11 @@ class TestCase(BaseTest):
         indexer += temp.directory
         indexer += self.tempdir
         assert len(indexer) == 3
-        indexer.add(text=analysis.WhitespaceTokenizer(util.Version.LUCENE_CURRENT, StringReader('?')), name=lucene.JArray_byte('{}'))
+        indexer.add(text=analysis.core.WhitespaceTokenizer(util.Version.LUCENE_CURRENT, StringReader('?')), name=util.BytesRef('{}'))
         indexer.commit()
-        value = indexer[next(indexer.docs('text', '?'))]['name']
-        assert value == '{}' or value.utf8ToString() == '{}'
+        assert indexer[next(indexer.docs('text', '?'))]['name'] == '{}'
         reader = engine.indexers.IndexReader(indexer.indexReader)
         assert reader[0].dict() == {} and reader.count('text', '?') == 1
-        assert len(reader.comparator('text')) == 4
         indexer.delete('text', '?')
         indexer.commit(merge=True)
         assert not indexer.hasDeletions()
@@ -221,7 +197,7 @@ class TestCase(BaseTest):
         indexer.commit(filters=True, spellcheckers=True)
         assert reader.refCount == 0
         assert list(indexer.filters) == list(indexer.spellcheckers) == ['amendment']
-        tokenizer = lambda reader: analysis.WhitespaceTokenizer(util.Version.LUCENE_CURRENT, reader)
+        tokenizer = lambda reader: analysis.core.WhitespaceTokenizer(util.Version.LUCENE_CURRENT, reader)
         doc['amendment'] = engine.Analyzer(tokenizer).tokens(doc['amendment'])
         doc['date'] = engine.Analyzer(tokenizer).tokens(doc['date']), 2.0
         scores = list(searcher.match(doc, 'text:congress', 'text:law', 'amendment:27', 'date:19*'))
@@ -245,9 +221,6 @@ class TestCase(BaseTest):
         assert sorted(map(int, indexer.terms('amendment'))) == range(1, 28)
         assert list(itertools.islice(indexer.terms('text', 'right'), 2)) == ['right', 'rights']
         assert list(indexer.terms('text', 'right*')) == ['right', 'rights']
-        if hasattr(search, 'WildcardTermEnum'):
-            with assertWarns(DeprecationWarning):
-                assert list(indexer.terms('text', 'right?')) == ['rights']
         assert list(indexer.terms('text', 'right', minSimilarity=0.5)) == ['eight', 'right', 'rights']
         word, count = next(indexer.terms('text', 'people', counts=True))
         assert word == 'people' and count == 8
@@ -282,7 +255,8 @@ class TestCase(BaseTest):
         assert math.isnan(hits.maxscore)
         hits = indexer.search('text:right', count=2, sort=sort, maxscore=True)
         assert hits.maxscore > max(hits.scores)
-        parser = lambda value: int((value.utf8ToString() if hasattr(value, 'utf8ToString') else value) or -1)
+        self.assertRaises(AssertionError, indexer.comparator, 'amendment', type=int, multi=True)
+        parser = lambda value: int(value.utf8ToString() or -1)
         comparator = indexer.comparator('amendment', type=int, parser=parser)
         hits = indexer.search('text:people').sorted(comparator.__getitem__)
         assert sorted(hits.ids) == list(hits.ids) and list(hits.ids) != ids
@@ -347,7 +321,7 @@ class TestCase(BaseTest):
         assert dict(indexer.termvector(id, 'text', counts=True))['persons'] == 2
         assert dict(indexer.positionvector(id, 'text'))['persons'] in ([3, 26], [10, 48])
         assert dict(indexer.positionvector(id, 'text', offsets=True))['persons'] == [(46, 53), (301, 308)]
-        analyzer = analysis.WhitespaceAnalyzer(util.Version.LUCENE_CURRENT)
+        analyzer = analysis.core.WhitespaceAnalyzer(util.Version.LUCENE_CURRENT)
         query = indexer.morelikethis(0, analyzer=analyzer)
         assert str(query) == 'text:united text:states'
         hits = indexer.search(query & engine.Query.prefix('article', ''))
@@ -356,12 +330,7 @@ class TestCase(BaseTest):
         assert str(indexer.morelikethis(0, minDocFreq=3, analyzer=analyzer)) == 'text:establish text:united text:states'
         assert str(indexer.morelikethis('jury', 'text', minDocFreq=4, minTermFreq=1, analyzer=analyzer)) == 'text:jury'
         assert str(indexer.morelikethis('jury', 'article', analyzer=analyzer)) == ''
-        try:
-            query = indexer.morelikethis('jury')
-        except lucene.JavaError:
-            pass
-        else:
-            assert str(query) == ''
+        self.assertRaises(lucene.JavaError, indexer.morelikethis, 'jury')
         assert indexer.suggest('missing', '') == list(indexer.correct('missing', '')) == []
         assert indexer.suggest('text', '')[:8] == ['shall', 'states', 'any', 'have', 'united', 'congress', 'state', 'constitution']
         assert indexer.suggest('text', 'con')[:2] == ['congress', 'constitution']
@@ -391,9 +360,12 @@ class TestCase(BaseTest):
             os.remove(filepath)
             open(filepath, 'w').close()
             self.assertRaises(OSError, engine.indexers.copy, commit, path)
+        self.assertRaises(AssertionError, indexer.check, self.tempdir)
         del indexer
         assert engine.Indexer(self.tempdir)
         assert not os.path.exists(os.path.join(self.tempdir, commit.segmentsFileName))
+        assert engine.IndexWriter.check(self.tempdir).clean
+        assert not engine.IndexWriter.check(self.tempdir, fix=True).numBadSegments
     
     def testAdvanced(self):
         "Large data set with hierarchical fields."
@@ -407,16 +379,8 @@ class TestCase(BaseTest):
                 location = '.'.join(doc[name] for name in ['state', 'county', 'city'])
                 indexer.add(doc, latitude=lat, longitude=lng, location=location)
         indexer.commit()
-        try:
-            names = indexer.names(indexed=True)
-        except AttributeError:
-            names = indexer.names('indexed', isIndexed=True)
-        assert set(['state', 'zipcode']) < set(names)
-        try:
-            names = indexer.names(indexed=False)
-        except AttributeError:
-            names = indexer.names('unindexed', isIndexed=False)
-        assert set(['latitude', 'longitude', 'county', 'city']) <= set(names)
+        assert sorted(indexer.names(indexed=True)) == ['state', 'state.county', 'state.county.city', 'zipcode']
+        assert sorted(indexer.names(indexed=False)) == ['city', 'county', 'latitude', 'longitude']
         states = list(indexer.terms('state'))
         assert states[0] == 'AK' and states[-1] == 'WY'
         counties = [term.split('.')[-1] for term in indexer.terms('state.county', 'CA', 'CA~')]
@@ -438,7 +402,8 @@ class TestCase(BaseTest):
         assert orange == 'CA.Orange' and facets[orange] > 10
         (field, facets), = indexer.facets(query, ('state.county', 'CA.*')).items()
         assert all(value.startswith('CA.') for value in facets) and set(facets) < set(indexer.filters[field])
-        assert set(indexer.grouping('state', count=1)) < set(indexer.grouping('state')) == set(states)
+        with assertWarns(DeprecationWarning):
+            assert set(indexer.grouping('state', count=1)) < set(indexer.grouping('state')) == set(states)
         grouper = indexer.grouping(field, query, sort=search.Sort(indexer.sorter(field)))
         assert len(grouper) == 2 and list(grouper) == [la, orange]
         for value, (name, count) in zip(grouper, grouper.facets(None)):
@@ -453,6 +418,16 @@ class TestCase(BaseTest):
         hits = next(grouper.groups(count=2, sort=search.Sort(indexer.sorter('zipcode')), scores=True))
         assert hits.value == 'CA.Los Angeles' and math.isnan(hits.maxscore) and len(hits) == 2
         assert all(hit.score > 0 and hit['zipcode'] > '90000' and hit['zipcode'] in hit.keys for hit in hits)
+        hits, = indexer.groupby(field, engine.Query.term('state', 'CA'), count=1)
+        assert hits.value == 'CA.Los Angeles' and len(hits) == 1 and hits.count > 100
+        grouping = engine.documents.GroupingSearch(field, sort=search.Sort(indexer.sorter(field)), cache=False, allGroups=True)
+        assert all(dict(grouping.facets(indexer.indexSearcher)).values())
+        assert len(grouping) == len(list(grouping)) > 100
+        assert set(grouping) > set(facets)
+        grouping.select()
+        hits, = grouping.groups(indexer.indexSearcher, engine.Query.term('state', 'CA'), count=1)
+        assert hits.value == 'CA.Alameda' and len(hits) == 1 and hits.count > 50
+        assert hits[0] == {}
         for count in (None, len(indexer)):
             hits = indexer.search(query, count=count, timeout=0.01)
             assert 0 <= len(hits) <= indexer.count(query) and hits.count in (None, len(hits)) and hits.maxscore in (None, 1.0)
@@ -477,8 +452,8 @@ class TestCase(BaseTest):
         for name, params in fixture.zipcodes.fields.items():
             indexer.set(name, **params)
         for name in ('longitude', 'latitude'):
-            indexer.set(name, engine.NumericField, store=True)
-        field = indexer.fields['tile'] = engine.PointField('tile', precision=15, step=2, store=True)
+            indexer.set(name, engine.NumericField, type=float, stored=True)
+        field = indexer.fields['tile'] = engine.PointField('tile', precision=15, numericPrecisionStep=2, stored=True)
         points = []
         for doc in fixture.zipcodes.docs():
             if doc['state'] == 'CA':
@@ -527,21 +502,23 @@ class TestCase(BaseTest):
     
     def testFields(self):
         "Custom fields."
-        self.assertRaises(lucene.JavaError, engine.Field, '', store='invalid')
-        self.assertRaises(AttributeError, engine.Field, '', omit='value')
-        self.assertRaises(lucene.JavaError, engine.Field, '', index=False)
-        field = engine.Field('', index=True, analyzed=True, omitNorms=True, termvector=True, withPositions=True, withOffsets=True)
+        self.assertRaises(lucene.InvalidArgsError, engine.Field, '', stored='invalid')
+        settings = {'indexed': False, 'docValueType': 'NUMERIC', 'indexOptions': 'DOCS_ONLY'}
+        field = engine.Field('', **settings)
+        assert field.settings == engine.Field('', **field.settings).settings == settings
+        with assertWarns(DeprecationWarning):
+            field = engine.NestedField('', store=True)
+        assert field.settings == {'indexed': True, 'tokenized': False, 'stored': True}
+        field = engine.Field('', omitNorms=True, storeTermVectors=True, storeTermVectorPositions=True, storeTermVectorOffsets=True)
         field, = field.items(' ')
-        attrs = 'indexed', 'tokenized', 'termVectorStored', 'storePositionWithTermVector', 'storeOffsetWithTermVector', 'omitNorms'
-        try:
-            assert all(getattr(field, attr) for attr in attrs)
-        except AttributeError:
-            attrs = 'indexed', 'tokenized', 'storeTermVectors', 'storeTermVectorPositions', 'storeTermVectorOffsets', 'omitNorms'
-            assert all(getattr(field.fieldType(), attr)() for attr in attrs)
+        attrs = 'indexed', 'tokenized', 'storeTermVectors', 'storeTermVectorPositions', 'storeTermVectorOffsets', 'omitNorms'
+        assert all(getattr(field.fieldType(), attr)() for attr in attrs)
         indexer = engine.Indexer(self.tempdir)
-        indexer.set('amendment', engine.MapField, func='{0:02d}'.format, store=True)
-        indexer.set('size', engine.MapField, func='{0:04d}'.format, store=True)
-        field = indexer.fields['date'] = engine.NestedField('Y-m-d', sep='-', store=True)
+        indexer.set('', store='yes', termvector='no')
+        assert indexer.fields.pop('').settings == {'indexed': True, 'stored': True}
+        indexer.set('amendment', engine.MapField, func='{0:02d}'.format, stored=True)
+        indexer.set('size', engine.MapField, func='{0:04d}'.format, stored=True)
+        field = indexer.fields['date'] = engine.NestedField('Y-m-d', sep='-', stored=True)
         for doc in fixture.constitution.docs():
             if 'amendment' in doc:
                 indexer.add(amendment=int(doc['amendment']), date=doc['date'], size=len(doc['text']))
@@ -570,7 +547,7 @@ class TestCase(BaseTest):
         assert list(hits.ids) == ids[:len(hits)]
         query = engine.Query.range('size', None, '1000')
         assert indexer.count(query) == len(sizes) - len(ids)
-        parser = lambda date: int((date.utf8ToString() if lucene.VERSION >= '4' else date).split('-')[0])
+        parser = lambda date: int(date.utf8ToString().split('-')[0])
         indexer.sorters['year'] = engine.SortField('Y-m-d', type=int, parser=parser)
         assert list(indexer.comparator('year'))[:10] == [1791] * 10
         cache = len(search.FieldCache.DEFAULT.cacheEntries)
@@ -591,10 +568,13 @@ class TestCase(BaseTest):
     
     def testNumeric(self):
         "Numeric fields."
+        nf, = engine.NumericField('temp').items(0.5)
+        assert nf.numericValue().doubleValue() == 0.5
         indexer = engine.Indexer(self.tempdir)
-        indexer.set('amendment', engine.NumericField, store=True)
-        indexer.set('date', engine.DateTimeField, store=True)
-        indexer.set('size', engine.NumericField, store=True, step=5)
+        indexer.set('amendment', engine.NumericField, type=int, stored=True)
+        indexer.set('date', engine.DateTimeField, stored=True)
+        with assertWarns(DeprecationWarning):
+            indexer.set('size', engine.NumericField, type=int, stored=True, step=5)
         for doc in fixture.constitution.docs():
             if 'amendment' in doc:
                 indexer.add(amendment=int(doc['amendment']), date=[tuple(map(int, doc['date'].split('-')))], size=len(doc['text']))
@@ -609,7 +589,7 @@ class TestCase(BaseTest):
         assert indexer.count(query) == 19
         query = field.range(datetime.date(1919, 1, 1), datetime.date(1921, 12, 31))
         hits = indexer.search(query)
-        assert [hit['amendment'] for hit in hits] == ['18', '19']
+        assert [hit['amendment'] for hit in hits] == [18, 19]
         assert [datetime.datetime.utcfromtimestamp(float(hit['date'])).year for hit in hits] == [1919, 1920]
         assert indexer.count(field.within(seconds=100)) == indexer.count(field.within(weeks=1)) == 0
         query = field.duration([2009], days=-100*365)
@@ -626,23 +606,21 @@ class TestCase(BaseTest):
         query = field.range(None, 1000)
         assert indexer.count(query) == len(sizes) - len(ids)
         self.assertRaises(OverflowError, list, field.items(-2**64))
-        nf, = field.items(0.5)
-        assert (nf.numericValue if hasattr(document, 'NumericField') else nf.numericValue()).doubleValue() == 0.5
         assert str(field.range(-2**64, 0)) == 'size:[* TO 0}'
         assert str(field.range(0, 2**64)) == 'size:[0 TO *}'
         assert str(field.range(0.5, None, upper=True)) == 'size:[0.5 TO *]'
-        for step, count in zip(range(0, 20, field.step), (26, 19, 3, 1)):
+        for step, count in zip(range(0, 20, field.numericPrecisionStep()), (26, 19, 3, 1)):
             sizes = list(indexer.numbers('size', step))
             assert len(sizes) == count and all(isinstance(size, int) for size in sizes)
             numbers = dict(indexer.numbers('size', step, type=float, counts=True))
             assert sum(numbers.values()) == len(indexer) and all(isinstance(number, float) for number in numbers)
         hit, = indexer.search(indexer.fields['amendment'].term(1))
-        assert hit['amendment'] == '1'
+        assert hit['amendment'] == 1
     
     def testHighlighting(self):
         "Highlighting text fragments."
         indexer = engine.Indexer()
-        indexer.set('text', store=True, termvector=True, withPositions=True, withOffsets=True)
+        indexer.set('text', stored=True, storeTermVectors=True, storeTermVectorPositions=True, storeTermVectorOffsets=True)
         for doc in fixture.constitution.docs():
             if 'amendment' in doc:
                 indexer.add(text=doc['text'])
@@ -672,7 +650,7 @@ class TestCase(BaseTest):
     
     def testNearRealTime(self):
         "Near real-time index updates."
-        indexer = engine.Indexer(version=util.Version.LUCENE_30, nrt=True)
+        indexer = engine.Indexer(version=util.Version.LUCENE_43, nrt=True)
         indexer.add()
         assert indexer.count() == 0 and not indexer.current
         indexer.refresh(filters=True)
@@ -686,7 +664,7 @@ class TestCase(BaseTest):
     def testFilters(self):
         "Custom filters."
         indexer = engine.Indexer()
-        indexer.set('name', store=True, index=True)
+        indexer.set('name', stored=True, indexed=True)
         for name in ('alpha', 'bravo', 'charlie'):
             indexer.add(name=name)
         indexer.commit()
@@ -696,7 +674,7 @@ class TestCase(BaseTest):
         filter.discard('bravo', 'charlie')
         assert filter.values == set(['alpha'])
         parallel = engine.ParallelIndexer('name')
-        parallel.set('priority', index=True)
+        parallel.set('priority', tokenized=False)
         for name in ('alpha', 'bravo', 'delta'):
             parallel.update(name, priority='high')
         parallel.commit()
