@@ -128,15 +128,16 @@ class TestCase(BaseTest):
             resource.get('/terms/x/y/missing')
         with assertRaises(httplib.HTTPException, httplib.NOT_FOUND):
             resource.get('/terms/x/y/docs/missing')
-        defaults = {'index': 'ANALYZED', 'store': 'NO', 'termvector': 'NO'}
+        defaults = {'indexed': True}
         response = resource.call('PUT', '/fields/text')
         assert response.getheader('etag') is None
         assert response.status == httplib.CREATED and response() == defaults
         response = resource.call('PUT', '/fields/text', {})
         assert response.status == httplib.OK and response() == defaults
-        assert resource.put('/fields/name', {'store': True, 'index': 'not_analyzed'})
+        with local.assertWarns(UserWarning):
+            assert resource.put('/fields/name', {'store': True, 'index': 'not_analyzed'})
         assert sorted(resource.get('/fields')) == ['name', 'text']
-        assert resource.get('/fields/text')['index'] == 'ANALYZED'
+        assert resource.get('/fields/text') == {'indexed': True}
         assert not resource.post('/docs', [{'name': 'sample', 'text': 'hello world'}])
         assert not resource.post('/docs')
         (directory, count), = resource.get('/').items()
@@ -164,8 +165,8 @@ class TestCase(BaseTest):
         assert resource.get('/docs/0') == resource.get('/docs/name/sample') == {'name': 'sample'}
         assert resource.get('/docs/0', fields='missing') == {'missing': None}
         assert resource.get('/docs/0', fields='', **{'fields.multi': 'missing'}) == {'missing': []}
-        assert resource.get('/terms') == ['name', 'text']
-        assert resource.get('/terms', option='unindexed', isIndexed=False) == []
+        assert resource.get('/terms') == resource.get('/terms', indexed='true') == ['name', 'text']
+        assert resource.get('/terms', indexed='false') == []
         assert resource.get('/terms/text') == ['hello', 'world']
         assert resource.get('/terms/text/world') == 1
         with assertRaises(httplib.HTTPException, httplib.BAD_REQUEST):
@@ -236,7 +237,7 @@ class TestCase(BaseTest):
             resource.put('/docs/name/sample', {'name': 'mismatched'})
         with assertRaises(httplib.HTTPException, httplib.CONFLICT):
             resource.put('/docs/missing/sample')
-        assert resource.put('/fields/name', {'store': True, 'index': False, 'omitNorms': True})
+        assert resource.put('/fields/name', {'stored': True, 'indexed': False, 'omitNorms': True})
         with assertRaises(httplib.HTTPException, httplib.CONFLICT):
             resource.put('/docs/name/sample')
         assert not resource.delete('/docs/missing/sample')
@@ -285,10 +286,7 @@ class TestCase(BaseTest):
         filepath = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'lupyne/server.py')
         assert subprocess.call((sys.executable, filepath, '-c', filepath), stderr=subprocess.PIPE)
         assert cherrypy.tree.mount(None)
-        try:
-            server.init(vmargs=None)
-        except ValueError:
-            pass
+        server.init(vmargs=None)
         self.assertRaises(AttributeError, server.start, config=True)
     
     def testBasic(self):
@@ -301,7 +299,7 @@ class TestCase(BaseTest):
         fields = resource.get('/fields')
         assert sorted(fields) == ['amendment', 'article', 'date', 'text']
         for field in fields:
-            assert sorted(resource.get('/fields/' + name)) == ['index', 'store', 'termvector']
+            assert resource.get('/fields/' + name)['indexed']
         resource.post('/docs', list(fixture.constitution.docs()))
         assert resource.get('/').values() == [35]
         resource.post('/update', {'spellcheckers': True, 'merge': 1})
@@ -332,6 +330,8 @@ class TestCase(BaseTest):
         assert sorted(counts) == docs and all(counts.values()) and sum(counts.values()) > len(counts)
         positions = dict(resource.get('/terms/text/people/docs/positions'))
         assert sorted(positions) == docs and list(map(len, positions.values())) == counts.values()
+        doc = resource.get('/docs/amendment/1', **{'fields.multi': 'text', 'fields.indexed': 'text'})
+        assert doc['text'][:3] == ['abridging', 'assemble', 'congress']
         doc, = resource.get('/search', q='amendment:1', fields='', **{'fields.indexed': 'article,amendment:int'})['docs']
         assert doc['amendment'] == 1 and not doc['article']
         result = resource.get('/search', **{'q.field': 'text', 'q': 'write "hello world"', 'spellcheck': 3})
@@ -432,7 +432,7 @@ class TestCase(BaseTest):
         writer = engine.IndexWriter(self.tempdir)
         writer.commit()
         self.start(self.ports[0], '-r', self.tempdir, **{'tools.validate.etag': False})
-        writer.set('zipcode', engine.NumericField, store=True)
+        writer.set('zipcode', engine.NumericField, type=int, stored=True)
         writer.fields['location'] = engine.NestedField('county.city')
         for doc in fixture.zipcodes.docs():
             if doc['state'] == 'CA':
